#ifndef OGL_HELPERS_H
#define OGL_HELPERS_H


#ifndef linux
#define WIN32_LEAN_AND_MEAN
#include <windows.h> // Needed for OpenGL on windows
#endif

#include <xsi_application.h>
#include <xsi_string.h>
#include <xsi_toolcontext.h>

//rescale 
const double GetViewScaleFactor( ToolContext &in_ctxt, const CVector3 &in_Pos, double in_dViewSize ) 
{
	CVector3 l_Offset;
	if ( in_ctxt.WorldToView( in_Pos, l_Offset ) == CStatus::OK )
	{
		// Displace point 1-pixel in view-space
		l_Offset.PutX( l_Offset.GetX() + 1.0/*pixels*/ );
		if ( in_ctxt.ViewToWorld( l_Offset, l_Offset ) == CStatus::OK )
		{
			l_Offset.SubInPlace( in_Pos );
			return l_Offset.GetLength() * in_dViewSize;
		}
	}
	return 0.0; // Projection failed
}

bool LogOpenGLErrors( const CString &extraInfo = CString() )
{
	bool l_bError = false;
	for ( GLenum error; (error = glGetError()) != GL_NO_ERROR; )
	{
		const char *msg = (const char *)gluErrorString( error );
		CString errorMsg = L"OpenGL error - ";
		errorMsg += ( msg ? msg : "unknown error" );
		if ( !extraInfo.IsEmpty() )
		{
			errorMsg += " (";
			errorMsg += extraInfo;
			errorMsg += ")";
		}
		Application().LogMessage( errorMsg, siWarningMsg );
		l_bError = true;
	}
	return l_bError;
}

void TermHeadlight( ToolContext &in_ctxt )
{
	GLint l_MaxLights = 8;
	GLfloat noColor[] = { 0.0, 0.0, 0.0, 0.0 };
	GLfloat ambientColor[] = { 0.0, 0.0, 0.0, 1.0 };
	glLightModelfv( GL_LIGHT_MODEL_AMBIENT, ambientColor );
	glGetIntegerv( GL_MAX_LIGHTS, &l_MaxLights );
	for (GLint i = 0; i < l_MaxLights; ++i )
	{
		glDisable( GL_LIGHT0 + i );
		glLightfv( GL_LIGHT0 + i, GL_DIFFUSE,  noColor );
		glLightfv( GL_LIGHT0 + i, GL_SPECULAR, noColor );
		glLightfv( GL_LIGHT0 + i, GL_AMBIENT,  noColor );
	}
}	
void InitHeadlight( ToolContext &in_ctxt )
{
	TermHeadlight( in_ctxt ); // Disable existing lights
	CVector3 l_Pos( -1, 0, -2 );
	in_ctxt.CameraToWorld( l_Pos, l_Pos );
	GLfloat lightColor[]   = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat ambientColor[] = { 0.2, 0.2, 0.2, 1.0 };
	GLfloat noColor[]      = { 0.0, 0.0, 0.0, 0.0 };
	GLfloat position[]     = { (GLfloat)l_Pos.GetX(), (GLfloat)l_Pos.GetY(), (GLfloat)l_Pos.GetZ(), 0.0 };
	glLightfv( GL_LIGHT0, GL_POSITION, position );
	glLightfv( GL_LIGHT0, GL_DIFFUSE,  lightColor );
	glLightfv( GL_LIGHT0, GL_SPECULAR, noColor );
	glLightf(  GL_LIGHT0, GL_SPOT_EXPONENT, 0.0 );
	glLightModelfv( GL_LIGHT_MODEL_AMBIENT, ambientColor );
	glEnable( GL_LIGHT0 );
}


#endif